--[[ 
 -- Copyright (C) 2006-2012 Rivello Multimedia Consulting.                    
 -- code [at] RivelloMultimediaConsulting [dot] com                                                  
 --                                                                      
 -- Permission is hereby granted, free of charge, to any person obtaining
 -- a copy of this software and associated documentation files (the      
 -- "Software"), to deal in the Software without restriction, including  
 -- without limitation the rights to use, copy, modify, merge, publish,  
 -- distribute, sublicense, and#or sell copies of the Software, and to   
 -- permit persons to whom the Software is furnished to do so, subject to
 -- the following conditions:                                            
 --                                                                      
 -- The above copyright notice and this permission notice shall be       
 -- included in all copies or substantial portions of the Software.      
 --                                                                      
 -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,      
 -- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF   
 -- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 -- IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR    
 -- OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 -- ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 -- OTHER DEALINGS IN THE SOFTWARE.                                      
--]]  
-- Marks the right margin of code *******************************************************************

--------------------------------------
-- Imports
--------------------------------------
local Widget = require ("widget");

--------------------------------------
-- Metadata
--------------------------------------
module (..., package.seeall)

--------------------------------------
-- Class Methods
--------------------------------------
-- PUBLIC STATIC


--------------------------------------
-- Class Properties
--------------------------------------
-- PUBLIC STATIC
 

--------------------------------------
-- Class
--------------------------------------
--[[ 
	This class is designed to...
--]] 
function DirectionalKeypad(aTable, aX_num, aY_num, aWidth_num, aHeight_num)
	
	--------------------------------------
	-- Properties
	--------------------------------------
	
	-- PUBLIC
	
	-- PRIVATE
	--[[ 
		This property is designed to...
	--]]
	local me = display.newGroup();
	
	--------------------------------------
	-- Constructor
	--------------------------------------
	--[[ 
		This is the constructor...
	--]]
	function me:constructor()
		
		--TRACE
		--print ("DirectionalKeypad:constructor()");
				
		--RETURN
		return me;
	end 
	 
	--[[ 
		This is the initialize.
	--]]
	function me:initialize(aX_num, aY_num, aWidth_num, aHeight_num)
		
		--TRACE
		
		--CREATE FOUR BUTTONS TO CONTROL LEFT/RIGHT/UP/DOWN OF THE GAME
	    local up_button 	= Widget.newButton{
	        id 				= "up_button",
	        label 			= "Up",
	        width 			= aWidth_num/4,
	        height 			= aHeight_num/4,
	        left 			= 0,
	        top 			= 0,
	        cornerRadius 	= 0,
	        onRelease 		= me.onKeyUp
	    }
	    local down_button 	= Widget.newButton{
	        id 				= "down_button",
	        label 			= "Down",
	        width 			= aWidth_num/4,
	        height 			= aHeight_num/4,
	        left 			= up_button.x - up_button.width/2,
	        top 			= up_button.y + up_button.height*.5,
	        cornerRadius	= 0,
	        onRelease 		= me.onKeyDown
	    }
	    local left_button 	= Widget.newButton{
	        id 				= "left_button", 
	        label 			= "Left",
	        width 			= aWidth_num/4,
	        height			= aHeight_num/4,
	        left 			= up_button.x - up_button.width*3/2,
	        top 			= up_button.y,
	        cornerRadius	= 0,
	        onRelease 		= me.onKeyLeft
	    }
	    local right_button 	= Widget.newButton{
	        id 				= "right_button",
	        label 			= "Right",
	        width 			= aWidth_num/4,
	        height 			= aHeight_num/4,
	        left 			= left_button.x + left_button.width*3/2,
	        top 			= left_button.y - left_button.height/2,
	        cornerRadius 	= 0,
	        onRelease 		= me.onKeyRight
	    }
	    local reset_button 	= Widget.newButton{
	        id 				= "reset_button",
	        label 			= "Reset",
	        width 			= aWidth_num/4,
	        height 			= aHeight_num/8,
	        left 			= down_button.x - down_button.width/2,
	        top 			= down_button.y + down_button.height,
	        cornerRadius 	= 0,
	        onRelease 		= me.onKeyReset
	    }

		--ADD TO LOCAL GROUP	 
	    me:insert(up_button.view);
	    me:insert(down_button.view);
	    me:insert(left_button.view);
	    me:insert(right_button.view);
	    me:insert(reset_button.view);
	    
	    --POSITION FULL GROUP
	    me.x = aX_num;
	    me.y = aY_num;
		
	end 

	--------------------------------------
	-- Methods
	-------------------------------------- 
	
	-- PUBLIC
	
	-- PRIVATE

	--------------------------------------
	-- Events
	-------------------------------------- 
	-- PUBLIC
	--[[ 
		-- KEY INPUT: Up
	--]]
	function me:onKeyUp( event )
		local event = { name="onKeyUp", target=me }
		me:dispatchEvent( event )
	end
	
	--[[ 
		-- KEY INPUT: Down
	--]]
	function me:onKeyDown( event )
		local event = { name="onKeyDown", target=me }
		me:dispatchEvent( event )
	end
	
	--[[ 
		-- KEY INPUT: Left
	--]]
	function me:onKeyLeft( event )
		local event = { name="onKeyLeft", target=me }
		me:dispatchEvent( event )
	end
	
	--[[ 
		-- KEY INPUT: Right
	--]]
	function me:onKeyRight( event )
		local event = { name="onKeyRight", target=me }
		me:dispatchEvent( event )
	end 
	
	--[[ 
		-- KEY INPUT: Reset
	--]]
	function me:onKeyReset( event )
		local event = { name="onKeyReset", target=me }
		me:dispatchEvent( event )
	end
	

	-- RETURN INSTANCE VIA CONSTRUCTOR
	me:initialize(aX_num, aY_num, aWidth_num, aHeight_num)
	return me:constructor();
end
