--[[ 
 -- Copyright (C) 2006-2012 Rivello Multimedia Consulting.                    
 -- code [at] RivelloMultimediaConsulting [dot] com                                                  
 --                                                                      
 -- Permission is hereby granted, free of charge, to any person obtaining
 -- a copy of this software and associated documentation files (the      
 -- "Software"), to deal in the Software without restriction, including  
 -- without limitation the rights to use, copy, modify, merge, publish,  
 -- distribute, sublicense, and#or sell copies of the Software, and to   
 -- permit persons to whom the Software is furnished to do so, subject to
 -- the following conditions:                                            
 --                                                                      
 -- The above copyright notice and this permission notice shall be       
 -- included in all copies or substantial portions of the Software.      
 --                                                                      
 -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,      
 -- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF   
 -- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 -- IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR    
 -- OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 -- ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 -- OTHER DEALINGS IN THE SOFTWARE.                                      
--]]  
-- Marks the right margin of code *******************************************************************

--------------------------------------
-- Imports
--------------------------------------
local Physics 			= require ("physics");

--------------------------------------
-- Metadata
--------------------------------------
module (..., package.seeall)

--------------------------------------
-- Class Methods
--------------------------------------
-- PUBLIC STATIC

--------------------------------------
-- Class Properties
--------------------------------------
-- PUBLIC STATIC


--------------------------------------
-- Class
--------------------------------------
--[[ 
	This class is designed to...
--]] 
function HitDetectionManager()
	
	--------------------------------------
	-- Properties
	--------------------------------------
	
	-- PRIVATE
	--[[ 
		Self-Reference
	--]]
	local me = display.newGroup();
	
	-- PUBLIC
	--[[ 
		COLLISION GROUP
	--]]
	me.COLLISION_GROUP_A = "COLLISION_GROUP_A";
	
	--[[ 
		COLLISION GROUP
	--]]
	me.COLLISION_GROUP_B = "COLLISION_GROUP_B";
	
	--------------------------------------
	-- Constructor
	--------------------------------------
	--[[ 
		This is the constructor...
	--]]
	function me:constructor()
		
		--TRACE
		--print ("HitDetectionManager.constructor()")
				
		--RETURN
		return me;
	end 
	 
	--[[ 
		One Time Setup
	--]]
	function me:initialize()
	
		--TRACE
		
		--SETUP
		Physics.start();
		--Physics.setDrawMode( "hybrid" ) 
		Physics.setGravity(0,0);
			
	end 
	
	--[[ 
		This is the constructor...
	--]]
	function me:start()
	
		--TRACE
		
		--SETUP
		
		--remove any existing events 
		Runtime:removeEventListener( "collision", 		me );
		Runtime:removeEventListener( "preCollision", 	me );
		Runtime:removeEventListener( "postCollision", 	me );
		 
		--add fresh events
		Runtime:addEventListener( "collision", 		me );
		Runtime:addEventListener( "preCollision", 	me );
		Runtime:addEventListener( "postCollision", 	me );
		
			
	end  
	
	
	--[[ 
		This is the constructor...
	--]]
	function me:stop()
	
		--TRACE
		
		--SETUP
		Runtime:removeEventListener( "collision", 		me );
		Runtime:removeEventListener( "preCollision", 	me );
		Runtime:removeEventListener( "postCollision", 	me );
		Physics.stop();
			
	end 


	--[[ 
		Add a new physics body.
		
		NOTE: There is alot of complexity to the filter system. Here is a very simple, effective use of it.
	--]]
	function me:addBody(aGroup, aGroupName_str)
		
		--DRAW
		local groupA_filter 	= { categoryBits = 1, maskBits = 2 } 
		local groupB_filter 	= { categoryBits = 2, maskBits = 1 } 
		 
		local groupA = { density=0.2, friction=0, bounce=0.95, filter=groupA_filter }
		local groupB = { density=0.2, friction=0, bounce=0.95, filter=groupB_filter }
		 
 		if aGroupName_str == me.COLLISION_GROUP_A then 
			Physics.addBody (aGroup, groupA);
		elseif aGroupName_str == me.COLLISION_GROUP_B then
			Physics.addBody (aGroup, groupB);
		else
			print ("ERROR!!!! Invalide GroupName")
		end
		
	end
	
	--[[ 
		Remove a new physics body.
	--]]
	function me:removeBody(aGroup)
		
		local wasRemoved_boolean = Physics.removeBody (aGroup)
		if not wasRemoved_boolean then
    		print( "HitDetectionManager.removeBody()" )
		end
		
		return wasRemoved_boolean;
	end

		  


	--------------------------------------
	-- Methods
	-------------------------------------- 
	
	-- PUBLIC

	 
	-- PRIVATE

	--------------------------------------
	-- Events
	-------------------------------------- 
	-- PUBLIC
	--[[ 
		EVENT: Before Collision
	--]]
	function me:preCollision( event )
		--print ("HitDetectionManager.preCollision()")
        local event = { name="preCollision", target=me }
		me:dispatchEvent( event )
	end
	
	--[[ 
		EVENT: During Collision
	--]]
	function me:collision( event )
		--
		if ( event.phase == "began" ) then
			--print ("HitDetectionManager.collision() began")
        elseif ( event.phase == "ended" ) then
			--print ("HitDetectionManager.collision() ended")
        end
        --
        local event = { name="collision", target=me, phase=event.phase }
		me:dispatchEvent( event )
	end
	
	--[[  
		EVENT: After Collision
	--]]
	function me:postCollision( event )
		--print ("HitDetectionManager.postCollision()")
        local event = { name="postCollision", target=me }
		me:dispatchEvent( event )
	end
	

	-- RETURN INSTANCE VIA CONSTRUCTOR
	me:initialize();
	return me:constructor();
end
