--[[ 
 -- Copyright (C) 2006-2012 Rivello Multimedia Consulting.                    
 -- code [at] RivelloMultimediaConsulting [dot] com                                                  
 --                                                                      
 -- Permission is hereby granted, free of charge, to any person obtaining
 -- a copy of this software and associated documentation files (the      
 -- "Software"), to deal in the Software without restriction, including  
 -- without limitation the rights to use, copy, modify, merge, publish,  
 -- distribute, sublicense, and#or sell copies of the Software, and to   
 -- permit persons to whom the Software is furnished to do so, subject to
 -- the following conditions:                                            
 --                                                                      
 -- The above copyright notice and this permission notice shall be       
 -- included in all copies or substantial portions of the Software.      
 --                                                                      
 -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,      
 -- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF   
 -- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 -- IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR    
 -- OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 -- ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 -- OTHER DEALINGS IN THE SOFTWARE.                                      
--]]  
-- Marks the right margin of code *******************************************************************

--------------------------------------
-- Imports
--------------------------------------
local AbstractGroup 	= require ("com.rmc.components.AbstractGroup");
local AssetManager 		= require ("com.rmc.projects.flyergame.managers.AssetManager");

--------------------------------------
-- Metadata
--------------------------------------
module (..., package.seeall)

--------------------------------------
-- Class Methods
--------------------------------------
-- PUBLIC STATIC


--------------------------------------
-- Class Properties
--------------------------------------
-- PUBLIC STATIC
 

--------------------------------------
-- Class
--------------------------------------
--[[ 
	This class is designed to...
--]] 
function Flyer(aTable, aX_num, aY_num, aSpeed_num)
	
	--------------------------------------
	-- Properties
	--------------------------------------
	-- PUBLIC
	
	-- PRIVATE
	--[[ 
		Self reference.
	--]]
	local me		 = AbstractGroup:AbstractGroup();
	
	--------------------------------------
	-- Constructor
	--------------------------------------
	--[[ 
		Only The Parent-Most class has a constructor
	--]]
	 
	--[[  
		This is the initialize.
	--]] 
	me.superInitialize = me.initialize;	--allow method override
	function me:initialize(aX_num, aY_num, aURL_str, aWidth_num, aHeight_num, aSpeed_num, aIsSpriteSheet_boolean)
		 
		--TRACE
		--print ("Flyer:initialize()");
		
		--SET SPEED	
		me.speed_num = aSpeed_num;
				
		--SUPER
		me:superInitialize(aX_num, aY_num, aURL_str, aWidth_num, aHeight_num, aIsSpriteSheet_boolean)
 
		
	end
	
	--------------------------------------
	-- Methods
	-------------------------------------- 
	
	-- PUBLIC
	
	--[[ 
		-- SPRITE MOVEMENT: Up
	--]]
	function me:doMoveUp( event )
		doMoveTo (me.x, me.y - me.speed_num)
	end

	--[[ 
		-- SPRITE MOVEMENT: Down
	--]]
	function me:doMoveDown( event )
		doMoveTo (me.x, me.y + me.speed_num)
	end
	
	--[[ 
		-- SPRITE MOVEMENT: Left
	--]]
	function me:doMoveLeft( event )
		doMoveTo (me.x - me.speed_num, me.y)
	end
	
	--[[ 
		-- SPRITE MOVEMENT: Right
	--]]
	function me:doMoveRight( event )
		doMoveTo (me.x + me.speed_num, me.y)
	end
	
	
	-- PRIVATE
	--[[ 
		-- SPRITE MOVEMENT: USE EASING TO MOVE TO A NEW X,Y
	--]]
	function doMoveTo( aX_num, aY_num )
		transition.to(me, {time=100, x=aX_num, y=aY_num, transition = easing.outQuad})
		--
		me.image:prepare("doAnimate")
		me.image:play()
	end

	--------------------------------------
	-- Events
	-------------------------------------- 
	-- PUBLIC
	--[[ 
		-- The Game Loop (Movement, Etc...)
	--]]
	function me:enterFrame( event )
		
		--MOVE POSITION
		me.x = me.x + me.xSpeed_num;

		--CORRECT POSITION IF OFFSCREEN
		if me.x - me.width > display.contentWidth then
			me.x = -me.width;
		elseif me.x + me.width < 0 then
			me.x = display.contentWidth;
		end
	    
	end
	

	-- RETURN INSTANCE VIA CONSTRUCTOR
	local assetManager = AssetManager:AssetManager();
	me:initialize(aX_num, aY_num, assetManager.IMAGE_FLYER,  69, 69, aSpeed_num, true );
	return me:constructor();
end
