--[[ 
 -- Copyright (C) 2006-2012 Rivello Multimedia Consulting.                    
 -- code [at] RivelloMultimediaConsulting [dot] com                                                  
 --                                                                      
 -- Permission is hereby granted, free of charge, to any person obtaining
 -- a copy of this software and associated documentation files (the      
 -- "Software"), to deal in the Software without restriction, including  
 -- without limitation the rights to use, copy, modify, merge, publish,  
 -- distribute, sublicense, and#or sell copies of the Software, and to   
 -- permit persons to whom the Software is furnished to do so, subject to
 -- the following conditions:                                            
 --                                                                      
 -- The above copyright notice and this permission notice shall be       
 -- included in all copies or substantial portions of the Software.      
 --                                                                      
 -- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,      
 -- EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF   
 -- MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 -- IN NO EVENT SHALL THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR    
 -- OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 -- ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 -- OTHER DEALINGS IN THE SOFTWARE.                                      
--]]  
-- Marks the right margin of code *******************************************************************

--------------------------------------
-- Imports
--------------------------------------
local AbstractScene 		= require ("com.rmc.scenes.AbstractScene");
local Storyboard 			= require ("storyboard");
local BackgroundImage 		= require ("com.rmc.projects.flyergame.groups.BackgroundImage");
local DirectionalKeypad 	= require ("com.rmc.components.controls.DirectionalKeypad");
local HitDetectionManager 	= require ("com.rmc.managers.HitDetectionManager");
local AssetManager 			= require ("com.rmc.projects.flyergame.managers.AssetManager");
local SoundManager 			= require ("com.rmc.managers.SoundManager");
local Biplane 				= require ("com.rmc.projects.flyergame.groups.Biplane");
local Blimp 				= require ("com.rmc.projects.flyergame.groups.Blimp");
local Flyer 				= require ("com.rmc.projects.flyergame.groups.Flyer");

--------------------------------------
-- Metadata
--------------------------------------
module (..., package.seeall)

--------------------------------------
-- Class Methods
--------------------------------------
-- PUBLIC STATIC
local STAGE_X 					= 0;
local STAGE_Y 					= 0;
local STAGE_LEFT_MARGIN 		= 20; --for text
local STAGE_WIDTH 				= display.contentWidth;
local STAGE_HEIGHT 				= display.contentHeight;
local FLYER_MOVEMENT_RADIUS 	= 60; 	--increase to move more per keypress
local STAGE_VICTORY_Y_POSITION 	= 30;
local ENEMY_MAX_MOVEMENT_RADIUS = 30; 	--increase to raise difficulty
  

--------------------------------------
-- Class Properties
--------------------------------------
-- PUBLIC STATIC


--------------------------------------
-- Class
--------------------------------------
--[[ 
	This class is designed to...
--]] 
function GameScene()
	
	--------------------------------------
	-- Properties
	--------------------------------------
	
	-- PUBLIC
	  
	-- PRIVATE
	--[[ 
		Self Reference
	--]]
	local me = AbstractScene:AbstractScene();
	
	--[[ 
		The manager of all physics-based hit-detection
	--]]
	local hitDetectionManager = HitDetectionManager:HitDetectionManager();
	
	--[[ 
		The manager of all sound
	--]]
	local soundManager = SoundManager:SoundManager();
	
	
	--[[ 
		Assets
	--]]
	local assetManager = AssetManager:AssetManager();
	local backgroundImage;
	local _score_text;
	local directionalKeypad;
	local biplane1;
	local biplane2;
	local blimp1;
	local blimp2;
	
	
	--------------------------------------
	-- Constructor
	--------------------------------------
	--[[ 
		Only The Parent-Most class has a constructor
	--]]
	 
	--[[ 
		This is the initialize.
	--]]
	me.superInitialize = me.initialize;		--allow method override
	function me:initialize()
		
		--TRACE
		print ("GameScene:initialize()");
		
		--SUPER
		me:superInitialize();
				
		--EVENTS
		
	end
	
	--------------------------------------
	-- Methods
	-------------------------------------- 
	
	-- PUBLIC
	--[[ 
		GAME END: VICTORY
	--]]
	function me:doEndGameWithWin()
		
		--TRACE
		print ("GameScene:doEndGameWithWin()");
		
		--SET SCORE
		me:doSetScore (100);
		
		--PLAY SOUND
		soundManager.playWinGameSound();
		
		--END GAME, STOP LISTENTING TO EVENTS
		me:doStopGame();
		
	end
	
	
	--[[ 
		GAME END: LOSS
	--]]
	function me:doEndGameWithLoss()
		
		--TRACE
		print ("GameScene:doEndGameWithLoss()");
		
		--SET SCORE
		me:doSetScore (-100);
		
		--PLAY SOUND
		soundManager.playLoseGameSound();
		
		--END GAME, STOP LISTENTING TO EVENTS
		me:doStopGame();
		
	end
	
	--[[ 
		GAME END
	--]]
	function me:doStopGame() 
		
		--TRACE
		print ("GameScene:doStopGame()");
		
  		--[[  
			CLEANUP: Immediate
		--]]
		Runtime:removeEventListener( "enterFrame", me );
		hitDetectionManager:addEventListener("postCollision", me );
		
		--[[ 
			CLEANUP: After Short delay (physics needs delay
		--]]
		
		--Use a delegate to map an event call back to a custom-named class method of 'me:postCollisionAfterDelay'
		local postCollisionAfterDelay_delegate = {}
		function postCollisionAfterDelay_delegate:timer( event )
		    me:postCollisionAfterDelay( event);
		end
		timer.performWithDelay(2000, postCollisionAfterDelay_delegate, 1 );

		
	end
	
	--[[  
		-- KEY INPUT: Right
	--]]
	function me:doMove( event )
		
		--CHECK FOR VICTORY
		soundManager.playMoveFlyerSound();
		--
		if flyer.y < STAGE_VICTORY_Y_POSITION then
			me:doEndGameWithWin();
		end
	end
	
    
	--[[  
		-- SET THE SCORE, UPDATE POSITION
	--]]
	function me:doSetScore( aScore_num )
	
		_score_text.text = "Score:"..aScore_num;
		_score_text.x = STAGE_X + STAGE_LEFT_MARGIN +  _score_text.width/2;
	
	end
	
	


	-- PRIVATE 
	

	--------------------------------------
	-- Events
	-------------------------------------- 
	
	-- PUBLIC
		
	--[[ 
		-- When touched by finger
	--]]
	function me:tap( event )
		--print"tap----------"
	    
	end
	
	--[[ 
		-- KEY INPUT: Up
	--]]
	function me:onKeyUp( event )
		flyer:doMoveUp();
		me:doMove();
	end
	
	--[[ 
		-- KEY INPUT: Down
	--]]
	function me:onKeyDown( event )
		flyer:doMoveDown();
		me:doMove();
	end
	
	--[[ 
		-- KEY INPUT: Left
	--]]
	function me:onKeyLeft( event )
		flyer:doMoveLeft();
		me:doMove();
	end
	
	--[[ 
		-- KEY INPUT: Right
	--]]
	function me:onKeyRight( event )
		--
		flyer:doMoveRight();
		me:doMove();
	end

	--[[ 
		-- KEY INPUT: Reset
	--]]
	function me:onKeyReset( event )
		me:doReturnToMainMenu();
	end
	
	
	--[[ 
		-- The Game Loop (Movement, Etc...)
	--]]
	function me:enterFrame( event )
		--
	    biplane1:enterFrame(event);
	   	biplane2:enterFrame(event);
	   	blimp1:enterFrame(event);
	   	blimp2:enterFrame(event);
	    
	end


	--[[ 
		-- "createScene" event is dispatched if scene's view does not exist
	--]]
	me.superCreateScene = me.createScene;	--allow method override
	function me:createScene( event )
	   
		--TRACE
		print ("GameScene:createScene()");
		
		--SUPER
		me:superCreateScene();
			   	   
		--CREATE LAYOUT
	  	local group = self.view;
	  	
		--STAGE POSITIONING PROPERTIES
		local screenHeight 		= STAGE_HEIGHT;
		local cloudHeight 		= STAGE_HEIGHT *0.15;
		local cloudlessArea		= screenHeight; 
	  	 
		--CREATE BG
		backgroundImage = BackgroundImage:BackgroundImage(assetManager.IMAGE_BACKGROUND);
		
		--CREATE SCORE TEXT
		_score_text = display.newText("", 170, 10, native.systemFont, 60) 
		_score_text:setTextColor(0,0,0)
		me:doSetScore(0);

		
		
	  	
	  	--CREATE CONTROLS
	  	directionalKeypad = DirectionalKeypad:DirectionalKeypad(STAGE_WIDTH - 175, STAGE_HEIGHT - 250, 300, 300);
	  	directionalKeypad:addEventListener("onKeyUp", me );
	  	directionalKeypad:addEventListener("onKeyDown", me );
	  	directionalKeypad:addEventListener("onKeyLeft", me );
	  	directionalKeypad:addEventListener("onKeyRight", me );
	  	directionalKeypad:addEventListener("onKeyReset", me );
	  	
		--PLACE SPRITES BASED BASED ON SCREEN % SO VARIOUS SCREEN SIZES WOULD WORK
		local trafficLane1Y     = cloudHeight + cloudlessArea*.1;
		local trafficLane2Y     = cloudHeight + cloudlessArea*.25;
		local trafficLane3Y     = cloudHeight + cloudlessArea*.4;
		local trafficLane4Y     = cloudHeight + cloudlessArea*.6; 
		local trafficLane5Y     = screenHeight - cloudHeight/2; 
		--
		biplane1 	  = Biplane:Biplane	(120, trafficLane1Y, ENEMY_MAX_MOVEMENT_RADIUS/3);
		blimp1 		  = Blimp:Blimp		(220, trafficLane2Y, -ENEMY_MAX_MOVEMENT_RADIUS/3);
		biplane2 	  = Biplane:Biplane	(330, trafficLane3Y, ENEMY_MAX_MOVEMENT_RADIUS/3);
		blimp2 		  = Blimp:Blimp		(440, trafficLane4Y, -ENEMY_MAX_MOVEMENT_RADIUS/2);
		flyer 		  = Flyer:Flyer		(500, trafficLane5Y, FLYER_MOVEMENT_RADIUS);


		--[[ 
			you can set an event this way too
			obj:addEventListener("touch", function(event) self:onCustomHandler(obj, event) end)
		--]]

			  
		--SETUP HIT-TESTING
		--ANY OF TYPE 'A' WILL HIT ONLY ANY OF TYPE 'B'
		group:insert(hitDetectionManager);	--non-visual, but is inserted so it dispatches events
		hitDetectionManager:start();
		hitDetectionManager:addBody (biplane1, 	hitDetectionManager.COLLISION_GROUP_A);
		hitDetectionManager:addBody (biplane2, 	hitDetectionManager.COLLISION_GROUP_A);
		hitDetectionManager:addBody (blimp1, 	hitDetectionManager.COLLISION_GROUP_A);
		hitDetectionManager:addBody (blimp2, 	hitDetectionManager.COLLISION_GROUP_A);
		hitDetectionManager:addBody (flyer, 	hitDetectionManager.COLLISION_GROUP_B);
		hitDetectionManager:addEventListener("collision", me );
		 
		--ADD TO LAYOUT  
		group:insert( backgroundImage );
		group:insert( _score_text );
		group:insert( directionalKeypad );
		group:insert( flyer );
		group:insert( biplane1 );
		group:insert( biplane2 );
		group:insert( blimp1 );
		group:insert( blimp2 );
		
	end 

	--[[ 
		EVENT: After Collision
	--]]	 
	function me:collision( event )
	
		me.doEndGameWithLoss();
	
	end 


	--[[ 
		EVENT: After Collision and a delay. The delay is needed to avoid a system crash
	--]]
	function me:postCollisionAfterDelay( event )
	
        --
        me:doReturnToMainMenu();

	end

	
	--[[ 
		RETURN TO MAIN MENU
	--]]
	function me:doReturnToMainMenu()
        
        print ("doReturnToMainMenu---")
        Storyboard.removeScene( "com.rmc.projects.flyergame.scenes.GameScene" )

	end

	
	--[[ 
		-- "enterScene" event is dispatched whenever scene transition has finished
	--]]
	me.superEnterScene = me.enterScene;		--allow method override
	function me:enterScene( event )
			
		--TRACE
		print ("GameScene:enterScene()");
				
		--SUPER
		me:superEnterScene();
		
		--EVENTS
		Runtime:addEventListener( "enterFrame", me );
		
	end
	
	--[[ 
		-- "exitScene" event is dispatched whenever before next scene's transition begins.
	--]]
	me.superExitScene = me.exitScene;	--allow method override
	function me:exitScene( event )
			
		--TRACE
		print ("GameScene:exitScene()");
				
		--SUPER
		me:superExitScene();
		
		--EVENTS
		--Runtime:removeEventListener( "tap", me );
		
	end
	
	--[[ 
		-- "destroyScene" event is dispatched before view is unloaded, which can be
		-- automatically unloaded in low memory situations, or explicitly via a call to
		-- storyboard.purgeScene() or storyboard.removeScene().
	--]]
	me.superDestroyScene = me.destroyScene;		--allow method override
	function me:destroyScene( event )
			 
		--TRACE
		print ("GameScene:destroyScene()");
		
		--
		me.superDestroyScene();
		 
		-- 
		Storyboard.purgeOnSceneChange = true
		Storyboard.gotoScene( "com.rmc.projects.flyergame.scenes.MainMenuScene", "crossFade", 500 )
		
		
	end
	
	
	
	
	-- RETURN INSTANCE VIA CONSTRUCTOR
	me:initialize();
	return me;
end




--------------------------------------
-- Instantiate
--------------------------------------
local GameScene = GameScene();
return GameScene;